@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2024 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2025 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -9,13 +9,11 @@
 
 #include <algorithm>
 #include <math.h>
-#include <wx/datetime.h>
 #include <wx/filename.h>
 #include <wx/log.h>
 #include <wx/msgdlg.h>
 #include <wx/txtstrm.h>
 #include <wx/wfstream.h>
-#include <yaml-cpp/yaml.h>
 
 #include "archive/GOArchive.h"
 #include "archive/GOArchiveFile.h"
@@ -34,6 +32,7 @@
 #include "files/GOOpenedFile.h"
 #include "files/GOStdFileName.h"
 #include "gui/dialogs/GOProgressDialog.h"
+#include "gui/dialogs/go-message-boxes.h"
 #include "gui/panels/GOGUIBankedGeneralsPanel.h"
 #include "gui/panels/GOGUICouplerPanel.h"
 #include "gui/panels/GOGUICrescendoPanel.h"
@@ -50,9 +49,9 @@
 #include "loader/cache/GOCache.h"
 #include "loader/cache/GOCacheWriter.h"
 #include "midi/GOMidi.h"
-#include "midi/GOMidiEvent.h"
 #include "midi/GOMidiPlayer.h"
 #include "midi/GOMidiRecorder.h"
+#include "midi/events/GOMidiEvent.h"
 #include "model/GOCoupler.h"
 #include "model/GODivisionalCoupler.h"
 #include "model/GOEnclosure.h"
@@ -61,11 +60,10 @@
 #include "model/GOSoundingPipe.h"
 #include "model/GOSwitch.h"
 #include "model/GOTremulant.h"
-#include "model/GOWindchest.h"
 #include "sound/GOSoundEngine.h"
 #include "sound/GOSoundReleaseAlignTable.h"
 #include "temperaments/GOTemperament.h"
-#include "yaml/go-wx-yaml.h"
+#include "yaml/GOYamlModel.h"
 
 #include "go_defs.h"
 
@@ -81,10 +79,7 @@
 static const wxString WX_ORGAN = wxT("Organ");
 static const wxString WX_GRANDORGUE_VERSION = wxT("GrandOrgueVersion");
 
-GOOrganController::GOOrganController(
-  GOConfig &config,
-  GOMidiDialogCreator *pMidiDialogCreator,
-  bool isAppInitialized)
+GOOrganController::GOOrganController(GOConfig &config, bool isAppInitialized)
   : GOEventDistributor(this),
     GOOrganModel(config),
     m_config(config),
@@ -106,7 +101,6 @@ GOOrganController::GOOrganController(
     m_b_customized(false),
     m_CurrentPitch(999999.0), // for enforcing updating the label first time
     m_OrganModified(false),
-    m_ChurchName(),
     m_ChurchAddress(),
     m_OrganBuilder(),
     m_OrganBuildDate(),
@@ -122,15 +116,14 @@ GOOrganController::GOOrganController(
     m_SampleSetId1(0),
     m_SampleSetId2(0),
     m_bitmaps(nullptr),
-    m_PitchLabel(this),
-    m_TemperamentLabel(this),
+    m_PitchLabel(*this),
+    m_TemperamentLabel(*this),
     m_MainWindowData(this, wxT("MainWindow")) {
   if (isAppInitialized) {
     // Load here objects that needs App (wx) to be loaded
     m_timer = new GOTimer();
     m_bitmaps = new GOBitmapCache(this);
   }
-  GOOrganModel::SetMidiDialogCreator(pMidiDialogCreator);
   GOOrganModel::SetModelModificationListener(this);
   m_setter = new GOSetter(this);
   m_pool.SetMemoryLimit(m_config.MemoryLimit() * 1024 * 1024);
@@ -145,15 +138,16 @@ GOOrganController::~GOOrganController() {
   m_tremulants.clear();
   m_ranks.clear();
   m_VirtualCouplers.Cleanup();
-  if (m_timer)
-    delete m_timer;
-  if (m_bitmaps)
-    delete m_bitmaps;
   GOOrganModel::Cleanup();
   GOOrganModel::SetModelModificationListener(nullptr);
-  GOOrganModel::SetMidiDialogCreator(nullptr);
   GOOrganModel::SetCombinationController(nullptr);
   m_elementcreators.clear();
+  // some elementcreator may reference to m_timer so we respect the deletion
+  // order
+  if (m_bitmaps)
+    delete m_bitmaps;
+  if (m_timer)
+    delete m_timer;
 }
 
 void GOOrganController::SetOrganModified(bool modified) {
@@ -171,7 +165,7 @@ void GOOrganController::OnIsModifiedChanged(bool modified) {
       = GetRootPipeConfigNode().GetPipeConfig().GetManualTuning();
 
     if (newPitch != m_CurrentPitch) {
-      m_PitchLabel.SetContent(wxString::Format(_("%f cent"), newPitch));
+      m_PitchLabel.SetContent(wxString::Format(_("%0.1f cent"), newPitch));
       m_CurrentPitch = newPitch;
     }
     // If the organ model is modified then the organ is also modified
@@ -209,7 +203,6 @@ void GOOrganController::ReadOrganFile(GOConfigReader &cfg) {
   /* load church info */
   cfg.ReadString(
     ODFSetting, WX_ORGAN, wxT("HauptwerkOrganFileFormatVersion"), false);
-  m_ChurchName = cfg.ReadStringTrim(ODFSetting, WX_ORGAN, wxT("ChurchName"));
   m_ChurchAddress = cfg.ReadString(ODFSetting, WX_ORGAN, wxT("ChurchAddress"));
   m_OrganBuilder
     = cfg.ReadString(ODFSetting, WX_ORGAN, wxT("OrganBuilder"), false);
@@ -325,12 +318,18 @@ void GOOrganController::ReadOrganFile(GOConfigReader &cfg) {
   for (unsigned i = 0; i < m_panels.size(); i++)
     m_panels[i]->Layout();
 
-  GetRootPipeConfigNode().SetName(GetChurchName());
+  const wxString &organName = GetOrganName();
+
+  GetRootPipeConfigNode().SetName(organName);
+  OnIsModifiedChanged(true);
   ReadCombinations(cfg);
   m_setter->OnCombinationsLoaded(GetCombinationsDir(), wxEmptyString);
+  ResetOrganModified();
 
   GOHash hash;
-  hash.Update(m_ChurchName.utf8_str(), strlen(m_ChurchName.utf8_str()));
+  const auto organNameUtf8 = organName.utf8_str();
+
+  hash.Update(organNameUtf8, strlen(organNameUtf8));
   GOHashType result = hash.getHash();
   m_SampleSetId1 = ((result.hash[0] & 0x7F) << 24)
     | ((result.hash[1] & 0x7F) << 16) | ((result.hash[2] & 0x7F) << 8)
@@ -340,8 +339,6 @@ void GOOrganController::ReadOrganFile(GOConfigReader &cfg) {
     | (result.hash[7] & 0x7F);
 }
 
-wxString GOOrganController::GetOrganHash() { return m_hash; }
-
 wxString GOOrganController::GenerateSettingFileName() {
   return m_config.OrganSettingsPath() + wxFileName::GetPathSeparator()
     + GOStdFileName::composeSettingFileName(GetOrganHash(), m_config.Preset());
@@ -627,111 +624,14 @@ wxString GOOrganController::Load(
 
 // const wxString &WX_CMB = wxT(".cmb");
 const wxString &WX_YAML = wxT("yaml");
-const char *const INFO = "info";
-const char *const CONTENT_TYPE = "content-type";
 const wxString WX_GRANDORGUE_COMBINATIONS = "GrandOrgue Combinations";
-const char *const ORGAN_NAME = "organ-name";
-const char *const GRANDORGUE_VERSION = "grandorgue-version";
-const char *const SAVED_TIME = "saved_time";
 
 wxString GOOrganController::ExportCombination(const wxString &fileName) {
-  wxString errMsg;
-  wxFileOutputStream fOS(fileName);
+  GOYamlModel::Out yamlOut(GetOrganName(), WX_GRANDORGUE_COMBINATIONS);
 
-  if (fOS.IsOk()) {
-    YAML::Node globalNode(YAML::NodeType::Map);
-    YAML::Node infoNode = globalNode[INFO];
-
-    infoNode[CONTENT_TYPE] = WX_GRANDORGUE_COMBINATIONS;
-    infoNode[ORGAN_NAME] = m_ChurchName;
-    infoNode[GRANDORGUE_VERSION] = APP_VERSION;
-    infoNode[SAVED_TIME] = wxDateTime::Now().Format();
-
-    globalNode << *m_setter;
-    globalNode << *m_DivisionalSetter;
-    YAML::Emitter outYaml;
-
-    outYaml << YAML::BeginDoc << globalNode;
-
-    static const uint8_t utf8bom[] = {0xEF, 0xBB, 0xBF};
-    if (
-      fOS.WriteAll(utf8bom, sizeof(utf8bom))
-      && fOS.WriteAll(outYaml.c_str(), outYaml.size()))
-      m_setter->OnCombinationsSaved(fileName);
-    else
-      errMsg.Printf(
-        wxT("Unable to write all the data to the file '%s'"), fileName);
-    fOS.Close();
-  } else
-    errMsg.Printf(wxT("Unable to open the file '%s' for writing"), fileName);
-  return errMsg;
-}
-
-/**
- * Check the churchName of the imported combination file. If it differs from the
- * current organ m_ChurchName then ask for the user
- * @param churchName the organ the combination file was saved of
- * @return true if the churchNames are the same or the user agree with importing
- *   the combination file
- */
-bool GOOrganController::IsToImportCombinationsFor(
-  const wxString &fileName, const wxString &churchName) const {
-  bool isToImport = true;
-
-  if (churchName != m_ChurchName) {
-    wxLogWarning(
-      _("This combination file '%s' was originally made for another organ "
-        "'%s'"),
-      fileName,
-      churchName);
-    isToImport = wxMessageBox(
-                   wxString::Format(
-                     _("This combination file '%s' was originally made for "
-                       "another organ '%s'. Importing it can cause various "
-                       "problems. Should it really be imported?"),
-                     fileName,
-                     churchName),
-                   _("Import Combinations"),
-                   wxYES_NO,
-                   NULL)
-      == wxYES;
-  }
-  return isToImport;
-}
-
-static std::vector<char> load_file_bytes(const wxString &filePath) {
-  wxFile file;
-  if (!file.Open(filePath)) {
-    throw wxString::Format(
-      _("Failed to open '%s': %s"), filePath, strerror(file.GetLastError()));
-  }
-  std::vector<char> content;
-  content.reserve(file.Length());
-  char buf[8 * 1024]; // 8 KiB
-  ssize_t bytesRead;
-  while ((bytesRead = file.Read(buf, sizeof(buf))) != 0) {
-    if (bytesRead == wxInvalidOffset) {
-      throw wxString::Format(
-        _("Failed to read '%s': %s"), filePath, strerror(file.GetLastError()));
-    }
-    content.insert(content.end(), &buf[0], &buf[bytesRead]);
-  }
-  return content;
-}
-
-static wxString load_file_text_with_encoding_detection(
-  const wxString &filePath) {
-  std::vector<char> content = load_file_bytes(filePath);
-  wxBOM detectedBOM = wxConvAuto::DetectBOM(&content[0], content.size());
-  if (detectedBOM != wxBOM_None && detectedBOM != wxBOM_Unknown) {
-    // We know what encoding was used for that file.
-    // wxConvAuto will use BOM to determine encoding and to decode file content.
-    // Note: newer GO versions export yaml files with UTF-8-BOM.
-    return wxString(&content[0], wxConvAuto(), content.size());
-  } else {
-    // Use encoding that was used in older GO versions (system default)
-    return wxString(&content[0], *wxConvCurrent, content.size());
-  }
+  yamlOut << *m_setter;
+  yamlOut << *m_DivisionalSetter;
+  return yamlOut.writeTo(fileName);
 }
 
 void GOOrganController::LoadCombination(const wxString &file) {
@@ -742,21 +642,15 @@ void GOOrganController::LoadCombination(const wxString &file) {
     const wxString fileExt = fileName.GetExt();
 
     if (fileExt == WX_YAML) {
-      wxString fileContent = load_file_text_with_encoding_detection(file);
-      // Note: wxScopedCharBuffer may point to internals of wxString above
-      // fileContent must not be destructed while fileContentInUtf8 is in use
-      wxScopedCharBuffer fileContentInUtf8 = fileContent.utf8_str();
-      YAML::Node cmbNode = YAML::Load(fileContentInUtf8.data());
-      YAML::Node cmbInfoNode = cmbNode[INFO];
-      const wxString contentType = cmbInfoNode[CONTENT_TYPE].as<wxString>();
+      GOYamlModel::In inYaml(GetOrganName(), file, WX_GRANDORGUE_COMBINATIONS);
 
-      if (contentType != WX_GRANDORGUE_COMBINATIONS)
-        throw wxString::Format(
-          _("The file '%s' is not a GrandOrgue Combination file"), file);
-      if (IsToImportCombinationsFor(
-            file, cmbInfoNode[ORGAN_NAME].as<wxString>())) {
-        cmbNode >> *m_setter;
-        cmbNode >> *m_DivisionalSetter;
+      if (is_to_import_to_this_organ(
+            GetOrganName(),
+            WX_GRANDORGUE_COMBINATIONS,
+            file,
+            inYaml.GetFileOrganName())) {
+        inYaml >> *m_setter;
+        inYaml >> *m_DivisionalSetter;
         m_setter->OnCombinationsLoaded(fileName.GetPath(), file);
       }
     } else {
@@ -768,24 +662,24 @@ void GOOrganController::LoadCombination(const wxString &file) {
       GOConfigReaderDB ini;
       ini.ReadData(odf_ini_file, CMBSetting, false);
       GOConfigReader cfg(ini);
-
-      wxString church_name
+      wxString fileOrganName
         = cfg.ReadString(CMBSetting, WX_ORGAN, wxT("ChurchName"));
-      if (!IsToImportCombinationsFor(file, church_name))
-        return;
 
-      wxString hash = odf_ini_file.getEntry(WX_ORGAN, wxT("ODFHash"));
-      if (hash != wxEmptyString)
-        if (hash != m_ODFHash) {
-          wxLogWarning(
-            _("The combination file does not exactly match the current ODF."));
-        }
-      /* skip informational items */
-      cfg.ReadString(CMBSetting, WX_ORGAN, wxT("ChurchAddress"), false);
-      cfg.ReadString(CMBSetting, WX_ORGAN, wxT("ODFPath"), false);
+      if (is_to_import_to_this_organ(
+            GetOrganName(), wxT("Organ Settings"), file, fileOrganName)) {
+        wxString hash = odf_ini_file.getEntry(WX_ORGAN, wxT("ODFHash"));
+        if (hash != wxEmptyString)
+          if (hash != m_ODFHash) {
+            wxLogWarning(_(
+              "The combination file does not exactly match the current ODF."));
+          }
+        /* skip informational items */
+        cfg.ReadString(CMBSetting, WX_ORGAN, wxT("ChurchAddress"), false);
+        cfg.ReadString(CMBSetting, WX_ORGAN, wxT("ODFPath"), false);
 
-      ReadCombinations(cfg);
-      m_setter->OnCombinationsLoaded(GetCombinationsDir(), wxEmptyString);
+        ReadCombinations(cfg);
+        m_setter->OnCombinationsLoaded(GetCombinationsDir(), wxEmptyString);
+      }
     }
     SetOrganModified();
   } catch (const wxString &error) {
@@ -804,6 +698,8 @@ void GOOrganController::LoadCombination(const wxString &file) {
 bool GOOrganController::CachePresent() { return wxFileExists(m_CacheFilename); }
 
 bool GOOrganController::UpdateCache(GOProgressDialog *dlg, bool compress) {
+  bool isOk = false;
+
   DeleteCache();
 
   /* Figure out the list of pipes to save */
@@ -812,38 +708,39 @@ bool GOOrganController::UpdateCache(GOProgressDialog *dlg, bool compress) {
   dlg->Setup(objectDistributor.GetNObjects(), _("Creating sample cache"));
 
   wxFileOutputStream file(m_CacheFilename);
-  GOCacheWriter writer(file, compress);
 
-  /* Save pipes to cache */
-  bool cache_save_ok = writer.WriteHeader();
+  if (file.IsOk()) {
+    GOCacheWriter writer(file, compress);
 
-  GOHashType hash = GenerateCacheHash();
-  if (!writer.Write(&hash, sizeof(hash)))
-    cache_save_ok = false;
+    /* Save pipes to cache */
+    bool isOk = writer.WriteHeader();
 
-  while (cache_save_ok) {
-    GOCacheObject *obj = objectDistributor.FetchNext();
+    GOHashType hash = GenerateCacheHash();
+    if (!writer.Write(&hash, sizeof(hash)))
+      isOk = false;
 
-    if (!obj)
-      break;
-    if (!obj->SaveCache(writer)) {
-      cache_save_ok = false;
-      wxLogError(
-        _("Save of %s to the cache failed"), obj->GetLoadTitle().c_str());
+    while (isOk) {
+      GOCacheObject *obj = objectDistributor.FetchNext();
+
+      if (!obj)
+        break;
+      if (!obj->SaveCache(writer)) {
+        isOk = false;
+        wxLogError(
+          _("Save of %s to the cache failed"), obj->GetLoadTitle().c_str());
+      }
+      if (!dlg->Update(objectDistributor.GetPos(), obj->GetLoadTitle())) {
+        writer.Close();
+        DeleteCache();
+        isOk = false;
+      }
     }
-    if (!dlg->Update(objectDistributor.GetPos(), obj->GetLoadTitle())) {
-      writer.Close();
+    writer.Close();
+    if (!isOk)
       DeleteCache();
-      return false;
-    }
-  }
-
-  writer.Close();
-  if (!cache_save_ok) {
-    DeleteCache();
-    return false;
-  }
-  return true;
+  } else
+    wxLogError(_("Opening the cache file %s failed"), m_CacheFilename);
+  return isOk;
 }
 
 void GOOrganController::DeleteCache() {
@@ -866,7 +763,7 @@ bool GOOrganController::Export(const wxString &cmb) {
 
   m_b_customized = true;
   cfg.WriteString(WX_ORGAN, wxT("ODFHash"), m_ODFHash);
-  cfg.WriteString(WX_ORGAN, wxT("ChurchName"), m_ChurchName);
+  cfg.WriteString(WX_ORGAN, wxT("ChurchName"), GetOrganName());
   cfg.WriteString(WX_ORGAN, wxT("ChurchAddress"), m_ChurchAddress);
   cfg.WriteString(WX_ORGAN, wxT("ODFPath"), GetODFFilename());
   if (m_ArchiveID != wxEmptyString)
@@ -943,8 +840,6 @@ void GOOrganController::AddPanel(GOGUIPanel *panel) {
   m_panels.push_back(panel);
 }
 
-const wxString &GOOrganController::GetChurchName() { return m_ChurchName; }
-
 const wxString &GOOrganController::GetChurchAddress() {
   return m_ChurchAddress;
 }
@@ -987,7 +882,7 @@ GOOrgan GOOrganController::GetOrganInfo() {
     GetODFFilename(),
     m_ArchiveID,
     m_ArchivePath,
-    GetChurchName(),
+    GetOrganName(),
     GetOrganBuilder(),
     GetRecordingDetails());
 }
@@ -999,7 +894,7 @@ const wxString GOOrganController::GetSettingFilename() {
 const wxString GOOrganController::GetCacheFilename() { return m_CacheFilename; }
 
 wxString GOOrganController::GetCombinationsDir() const {
-  return wxFileName(m_config.OrganCombinationsPath(), m_ChurchName)
+  return wxFileName(m_config.OrganCombinationsPath(), GetOrganName())
     .GetFullPath();
 }
 
@@ -1137,7 +1032,7 @@ void GOOrganController::SetTemperament(const GOTemperament &temperament) {
     m_ranks[k]->SetTemperament(temperament);
 }
 
-void GOOrganController::SetTemperament(wxString name) {
+void GOOrganController::SetTemperament(const wxString &name) {
   const GOTemperament &temperament
     = m_config.GetTemperaments().GetTemperament(name);
   m_Temperament = temperament.GetName();
